from collections.abc import Iterable
import importlib
import logging

from celery.schedules import crontab
from django.conf import settings
from django.utils import timezone
from django.db import transaction
from django.db.models import signals
from django.utils.module_loading import autodiscover_modules

from django_datawatch.defaults import defaults

logger = logging.getLogger(__name__)


class DatawatchHandler(object):
    def __init__(self):
        self._registered_checks = {}
        self._related_models = dict()
        self._backend = None

    def autodiscover_checks(self, module_name='checks'):
        autodiscover_modules(module_name)

    def register(self, check_class):
        slug = self.get_slug(check_class.__module__, check_class.__name__)
        self._registered_checks[slug] = check_class
        check = check_class()

        # register delete signal receiver if check is model based
        if check.model_class is not None:
            signals.post_delete.connect(
                delete_results,
                sender=check.model_class,
                dispatch_uid=f'django_datawatch_{slug}',
            )

        # register update
        if check.trigger_update is not None:
            for keyword, model in check.trigger_update.items():
                method_name = 'get_%s_payload' % keyword
                if not hasattr(check, method_name):
                    logger.warning(
                        'Update trigger "%s" defined without .%s()',
                        keyword, method_name)
                    continue

                # listen to updates on all trigger models
                model_uid = make_model_uid(model)
                datawatch._related_models.setdefault(model_uid, list())
                if check_class not in datawatch._related_models[model_uid]:
                    signals.post_save.connect(
                        run_checks,
                        sender=model,
                        dispatch_uid=f'django_datawatch_{slug}_{keyword}',
                    )
                    datawatch._related_models[model_uid].append(check_class)

        return check_class

    def get_all_registered_checks(self):
        return self._registered_checks.values()

    def get_all_registered_check_slugs(self):
        return self._registered_checks.keys()

    def get_check_class(self, slug):
        if slug in self._registered_checks:
            return self._registered_checks[slug]
        return None

    def get_checks_for_related_model(self, model):
        model_uid = make_model_uid(model)
        if model_uid in self._related_models:
            return self._related_models[model_uid]
        return None

    def get_checks_for_model(self, model):
        check_classes = list()
        for check_class in datawatch.get_all_registered_checks():
            if check_class.model_class == model:
                check_classes.append(check_class)
        return check_classes

    def get_slug(self, module, class_name):
        return u'{}.{}'.format(module, class_name)

    def get_backend(self):
        if self._backend is None:
            backend_module = importlib.import_module(
                getattr(settings, 'DJANGO_DATAWATCH_BACKEND',
                        defaults['BACKEND']))
            self._backend = backend_module.Backend()
        return self._backend

    def delete_results(self, sender, instance, db_alias=None):
        from django_datawatch.models import Result
        for check_class in datawatch.get_checks_for_model(model=sender):
            check = check_class()
            identifier = check.get_identifier(instance)
            Result.objects.using(db_alias).filter(slug=check.slug, identifier=identifier).delete()

    def update_related(self, sender, instance, db_alias=None):
        checks = datawatch.get_checks_for_related_model(sender) or []
        for check_class in checks:
            check = check_class()
            backend = datawatch.get_backend()
            model_uid = make_model_uid(instance.__class__)
            mapping = check.get_trigger_update_uid_map()

            if model_uid not in mapping:
                return

            if not hasattr(check, mapping[model_uid]):
                return

            payloads = getattr(check, mapping[model_uid])(instance)
            if not isinstance(payloads, Iterable):
                payloads = [payloads]

            for payload in payloads:
                if not payload:
                    continue

                # work around lambdas binding to the loop scope, see https://rules.sonarsource.com/python/RSPEC-1515
                def execute_backend_run(slug=check.slug, identifier=check.get_identifier(payload)):
                    backend.run(slug=slug, identifier=identifier, run_async=True)

                transaction.on_commit(execute_backend_run, using=db_alias)


datawatch = DatawatchHandler()


class Scheduler(object):
    def run_checks(self, force=False, slug=None):
        """
        :param force: <bool> force all registered checks to be executed
        :return:
        """
        checks = datawatch.get_all_registered_checks()
        last_executions = self.get_last_executions()

        for check_class in checks:
            check = check_class()

            # only update a single slug if requested
            if slug and check.slug != slug:
                continue

            # check is not meant to be run periodically
            if check_class.run_every is None:
                continue

            # schedule defined in an invalid format
            if not isinstance(check_class.run_every, crontab):
                logger.warning('run_every must be an instance of crontab')
                continue

            # shall the check be run again?
            if not force and check.slug in last_executions:
                last_execution = last_executions[check.slug]
                if timezone.is_aware(last_execution):
                    last_execution = last_execution.astimezone(timezone.get_current_timezone())
                if not check_class.run_every.is_due(last_run_at=last_execution).is_due:
                    continue

            # enqueue the check and save execution state
            check.run()

    def get_last_executions(self):
        from django_datawatch.models import CheckExecution
        return dict([(obj.slug, obj.last_run)
                     for obj in CheckExecution.objects.all()])


def make_model_uid(model):
    """
    Returns an uid that will identify the given model class.

    :param model: model class
    :return: uid (string)
    """
    return "%s.%s" % (model._meta.app_label, model.__name__)


def delete_results(sender, instance, using, **kwargs):
    if not getattr(settings, 'DJANGO_DATAWATCH_RUN_SIGNALS',
                   defaults['RUN_SIGNALS']):
        return
    datawatch.delete_results(sender, instance, using)


def run_checks(sender, instance, created, raw, using, **kwargs):
    """
    Re-execute checks related to the given sender model, only for the
    updated instance.

    :param sender: model
    :param kwargs:
    """
    if not getattr(settings, 'DJANGO_DATAWATCH_RUN_SIGNALS',
                   defaults['RUN_SIGNALS']):
        return
    try:
        datawatch.update_related(sender, instance, using)
    except Exception as e:
        logger.exception(e)



from django.urls import path

from django_datawatch import views

urlpatterns = [
    path('', views.DashboardView.as_view(), name='django_datawatch_index'),
    path('result/<int:pk>/', views.ResultView.as_view(), name='django_datawatch_result'),
    path('result/<int:pk>/acknowledge/', views.ResultAcknowledgeView.as_view(),
        name='django_datawatch_result_acknowledge'),
    path('result/<int:pk>/config/', views.ResultConfigView.as_view(), name='django_datawatch_result_config'),
    path('result/<int:pk>/refresh/', views.ResultRefreshView.as_view(), name='django_datawatch_result_refresh'),
]



import json

from dateutil import relativedelta
from django.utils import timezone
from django.conf import settings
from django.db import models
from django.core.exceptions import ValidationError
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _
from django_extensions.db.fields.json import JSONField
from model_utils.choices import Choices
from model_utils.models import TimeStampedModel

from django_datawatch.querysets import CheckExecutionQuerySet
from .datawatch import datawatch
from .querysets import ResultQuerySet


class AlreadyAcknowledged(Exception):
    pass


class Result(TimeStampedModel):
    STATUS = Choices((0, 'unknown', _('Unknown')),
                     (1, 'ok', _('OK')),
                     (2, 'warning', _('Warning')),
                     (3, 'critical', _('Critical')))

    slug = models.TextField(verbose_name=_('Module slug'))
    identifier = models.CharField(max_length=256, verbose_name=_('Identifier'))

    status = models.IntegerField(choices=STATUS,
                                 default=STATUS.unknown, verbose_name=_('Status'))
    data = JSONField(blank=True, default=dict, verbose_name=('Data'))
    config = JSONField(blank=True, default=dict, verbose_name=_('Configuration'))

    payload_description = models.TextField(verbose_name=_('Payload description'))

    acknowledged_by = models.ForeignKey(to=settings.AUTH_USER_MODEL, null=True, blank=True,
                                        verbose_name=_('Acknowledged by'),
                                        related_name='acknowledged_by', on_delete=models.CASCADE)
    acknowledged_at = models.DateTimeField(null=True, blank=True, verbose_name=_('Acknowledged at'))
    acknowledged_until = models.DateTimeField(null=True, blank=True, verbose_name=_('Acknowledged until'))
    acknowledged_reason = models.TextField(blank=True, verbose_name=_('Acknowledge reason'))

    assigned_users = models.ManyToManyField(to=settings.AUTH_USER_MODEL, through='ResultAssignedUser', related_name='assigned_results', blank=True, verbose_name=_('Assigned users'))
    assigned_groups = models.ManyToManyField(to='auth.Group', through='ResultAssignedGroup', related_name='assigned_groups', blank=True,
                                             verbose_name=_('Assigned groups'))

    objects = ResultQuerySet.as_manager()

    class Meta:
        unique_together = ('slug', 'identifier')
        permissions = (('view', 'Can view results dashboard and details'), ('acknowledge', 'Can acknowledge results'),
                       ('config', 'Can change the configuration for results'), ('refresh', 'Can refresh results'))

    def acknowledge(self, user, days, reason=None, commit=True):
        # calculate end of requested acknowledgement
        acknowledged_until = timezone.now() + relativedelta.relativedelta(days=days)

        # check that we're not accidentally overriding the current setup
        if self.status in (self.STATUS.warning, self.STATUS.critical):
            if self.is_acknowledged() and self.acknowledged_until > acknowledged_until:
                raise AlreadyAcknowledged()
        self.acknowledged_at = timezone.now()
        self.acknowledged_by = user
        self.acknowledged_until = acknowledged_until
        self.acknowledged_reason = reason or ''
        if commit:
            self.save(update_fields=['acknowledged_at', 'acknowledged_by', 'acknowledged_until', 'acknowledged_reason'])

    def is_acknowledged(self):
        return self.acknowledged_until and self.acknowledged_until >= timezone.now()

    def __str__(self):
        return self.slug

    def get_check_instance(self):
        return datawatch.get_check_class(self.slug)()

    def get_payload(self):
        return self.get_check_instance().get_payload(self.identifier)

    def get_formatted_data(self):
        return datawatch.get_check_class(self.slug)().format_result_data(self)

    def latest_status(self, status):
        return self.status_history.filter(to_status=status).order_by('-created').first()

    @cached_property
    def latest_unknown(self):
        return self.latest_status(self.STATUS.unknown)

    @cached_property
    def latest_ok(self):
        return self.latest_status(self.STATUS.ok)

    @cached_property
    def latest_warning(self):
        return self.latest_status(self.STATUS.warning)

    @cached_property
    def latest_critical(self):
        return self.latest_status(self.STATUS.critical)

    def config_formatted(self):
        return json.dumps(self.get_check_instance().get_config(payload=self.get_payload()), indent=4)


class ResultStatusHistory(TimeStampedModel):
    result = models.ForeignKey(Result, models.CASCADE, 'status_history', 'status_history', verbose_name=_('Result'))
    from_status = models.IntegerField(choices=Result.STATUS, verbose_name=_('From status'), null=True)
    to_status = models.IntegerField(choices=Result.STATUS, verbose_name=_('To status'))

    class Meta:
        verbose_name = _('Result status history')
        verbose_name_plural = _('Result status history')


class ResultAssignedGroup(models.Model):
    result = models.ForeignKey(Result, on_delete=models.CASCADE, verbose_name=_('Result'))
    group = models.ForeignKey(to='auth.Group', verbose_name=_('Group'), on_delete=models.CASCADE)

    class Meta:
        verbose_name = _('Result assigned group')
        verbose_name_plural = _('Result assigned groups')
        constraints = [models.UniqueConstraint(fields=['result', 'group'], name='unique_result_assigned_group')]

    def validate_unique(self, exclude=None):
        if (
            ResultAssignedGroup.objects.filter(result_id=self.result_id, group_id=self.group_id)
            .exclude(pk=self.pk)
            .exists()
        ):
            raise ValidationError({"group": _("Group must be unique across the result")})
        super().validate_unique(exclude)

class ResultAssignedUser(models.Model):
    result = models.ForeignKey(Result, on_delete=models.CASCADE, verbose_name=_('Result'))
    user = models.ForeignKey(to=settings.AUTH_USER_MODEL, verbose_name=_('User'), on_delete=models.CASCADE)

    class Meta:
        verbose_name = _('Result assigned user')
        verbose_name_plural = _('Result assigned users')
        constraints = [models.UniqueConstraint(fields=['result', 'user'], name='unique_result_assigned_user')]

    def validate_unique(self, exclude=None):
        if (
            ResultAssignedUser.objects.filter(result_id=self.result_id, user_id=self.user_id)
            .exclude(pk=self.pk)
            .exists()
        ):
            raise ValidationError({"user": _("User must be unique across the result")})
        super().validate_unique(exclude)

class CheckExecution(models.Model):
    slug = models.TextField(verbose_name=_('Check module slug'), unique=True)
    last_run = models.DateTimeField(verbose_name=_('Last run'))

    objects = CheckExecutionQuerySet.as_manager()

    def __str__(self):
        return '%s on %s' % (self.slug, self.last_run)



from django.core.management.base import BaseCommand

from django_datawatch.datawatch import datawatch


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument(
            '--slug',
            dest='slug',
            default=None,
            help='Slug of check to refresh, all checks will be refreshed if slug is not provided',
        )

    def handle(self, slug, *args, **options):
        # refresh single check
        if slug:
            self.refresh(slug)
            return

        # refresh all
        checks = datawatch.get_all_registered_checks()
        for check_class in checks:
            check = check_class()
            self.refresh(check.slug)

    def refresh(self, slug):
        backend = datawatch.get_backend()
        backend.refresh(slug=slug, run_async=True)



import logging

from django.core.management.base import BaseCommand

from django_datawatch.tasks import datawatch_cleanup

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Delete all results and executions for removed checks."

    def handle(self, *args, **options):
        datawatch_cleanup.apply()



# Generated by Django 4.2.9 on 2024-01-16 00:05

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('django_datawatch', '0003_resultstatushistory'),
    ]

    operations = [
        migrations.CreateModel(
            name='ResultAssignedUser',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('result', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_datawatch.result', verbose_name='Result')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'Result assigned user',
                'verbose_name_plural': 'Result assigned users',
            },
        ),
        migrations.CreateModel(
            name='ResultAssignedGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='auth.group', verbose_name='Group')),
                ('result', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_datawatch.result', verbose_name='Result')),
            ],
            options={
                'verbose_name': 'Result assigned group',
                'verbose_name_plural': 'Result assigned groups',
            },
        ),
        migrations.AddField(
            model_name='result',
            name='assigned_groups',
            field=models.ManyToManyField(blank=True, related_name='assigned_groups', through='django_datawatch.ResultAssignedGroup', to='auth.group', verbose_name='Assigned groups'),
        ),
        migrations.AddField(
            model_name='result',
            name='assigned_users',
            field=models.ManyToManyField(blank=True, related_name='assigned_results', through='django_datawatch.ResultAssignedUser', to=settings.AUTH_USER_MODEL, verbose_name='Assigned users'),
        ),
        migrations.AddConstraint(
            model_name='resultassigneduser',
            constraint=models.UniqueConstraint(fields=('result', 'user'), name='unique_result_assigned_user'),
        ),
        migrations.AddConstraint(
            model_name='resultassignedgroup',
            constraint=models.UniqueConstraint(fields=('result', 'group'), name='unique_result_assigned_group'),
        ),
    ]



from unittest import mock

from django.test.testcases import TestCase, override_settings

from django_datawatch.base import BaseCheck
from django_datawatch.datawatch import datawatch, delete_results
from django_datawatch.models import Result
from django_datawatch.querysets import ResultQuerySet


@datawatch.register
class CheckPostDelete(BaseCheck):
    model = Result

    def get_identifier(self, payload):
        return payload.pk

    def check(self, payload):
        return payload


class PostDeleteTestCase(TestCase):
    @override_settings(DJANGO_DATAWATCH_RUN_SIGNALS=True)
    @mock.patch('django_datawatch.datawatch.datawatch.delete_results')
    def test_setting_run_signals_true(self, mock_delete_results):
        delete_results(sender=None, instance=None, using=None)
        self.assertTrue(mock_delete_results.called)

    @override_settings(DJANGO_DATAWATCH_RUN_SIGNALS=False)
    @mock.patch('django_datawatch.datawatch.datawatch.delete_results')
    def test_setting_run_signals_false(self, mock_delete_results):
        delete_results(sender=None, instance=None, using=None)
        self.assertFalse(mock_delete_results.called)

    @override_settings(DJANGO_DATAWATCH_RUN_SIGNALS=True)
    @mock.patch('django_datawatch.datawatch.DatawatchHandler.get_checks_for_model')
    def test_update_related_calls_backend(self, mock_get_checks_for_model):
        # mock the list of checks
        mock_get_checks_for_model.return_value = [CheckPostDelete]

        # mock the manager
        with mock.patch.object(Result, "objects", spec=Result.objects) as mock_manager:
            mock_manager.using.return_value = mock_manager
            manager_filtered = mock.Mock(spec=ResultQuerySet)
            mock_manager.filter.return_value = manager_filtered

            # test if delete has been called
            datawatch.delete_results(sender=Result, instance=Result(pk=1))
            mock_manager.filter.assert_called_with(slug=CheckPostDelete().slug, identifier=1)
            manager_filtered.delete.assert_called_with()



from django.contrib import admin

from django_datawatch.models import Result, CheckExecution, ResultAssignedGroup, ResultAssignedUser, ResultStatusHistory


class ResultAssignedGroupInline(admin.TabularInline):
    model = ResultAssignedGroup
    extra = 0

class ResultAssignedUserInline(admin.TabularInline):
    model = ResultAssignedUser
    extra = 0

@admin.register(Result)
class CheckAdmin(admin.ModelAdmin):
    list_display = ('slug', 'identifier', 'status')
    readonly_fields = ('created', 'modified')
    search_fields = ('slug', 'identifier', 'payload_description')
    list_filter = ('status', 'slug', 'assigned_groups')
    inlines = [ResultAssignedGroupInline, ResultAssignedUserInline]

@admin.register(CheckExecution)
class CheckExecutionAdmin(admin.ModelAdmin):
    list_display = ('slug', 'last_run')
    search_fields = ('slug',)


@admin.register(ResultStatusHistory)
class ResultStatusHistory(admin.ModelAdmin):
    list_display = (
        'result',
        'from_status',
        'to_status',
        'created',
    )
    autocomplete_fields = ('result',)
    ordering = ('result', 'created')
    list_select_related = ['result']



from django_datawatch.backends.base import BaseBackend
from django_datawatch.tasks import django_datawatch_enqueue, \
    django_datawatch_run, django_datawatch_refresh


class Backend(BaseBackend):
    """
    A wrapper backend to execute tasks asynchronously in celery
    """
    def enqueue(self, slug, run_async=True):
        kwargs = dict(kwargs=dict(slug=slug))
        if run_async:
            django_datawatch_enqueue.apply_async(**kwargs)
        else:
            django_datawatch_enqueue.apply(**kwargs)

    def refresh(self, slug, run_async=True):
        kwargs = dict(kwargs=dict(slug=slug))
        if run_async:
            django_datawatch_refresh.apply_async(**kwargs)
        else:
            django_datawatch_refresh.apply(**kwargs)

    def run(self, slug, identifier, run_async=True, user_forced_refresh=False):
        kwargs = dict(kwargs=dict(slug=slug, identifier=identifier, user_forced_refresh=user_forced_refresh))
        if run_async:
            django_datawatch_run.apply_async(**kwargs)
        else:
            django_datawatch_run.apply(**kwargs)



# Generated by Django 4.2.9 on 2024-01-16 00:06

from django.db import migrations

from django.core.paginator import Paginator

def migrate_new_assigned_fields(apps, schema_editor):
    Result = apps.get_model('django_datawatch', 'Result')
    ResultAssignedGroup = apps.get_model('django_datawatch', 'ResultAssignedGroup')
    ResultAssignedUser = apps.get_model('django_datawatch', 'ResultAssignedUser')

    # Paginate the queryset to avoid memory issues
    paginator = Paginator(
        Result.objects.order_by('pk').only('assigned_to_user', 'assigned_to_group'),
        5000,
    )

    for page_number in paginator.page_range:
        page = paginator.page(page_number)
        group_instances = []
        user_instances = []

        for result in page.object_list:
            if result.assigned_to_group:
                group_instances.append(ResultAssignedGroup(
                    result_id=result.pk,
                    group_id=result.assigned_to_group.pk,
                ))
            if result.assigned_to_user:
                user_instances.append(ResultAssignedUser(
                    result_id=result.pk,
                    user_id=result.assigned_to_user.pk,
                ))

        ResultAssignedGroup.objects.bulk_create(group_instances)
        ResultAssignedUser.objects.bulk_create(user_instances)


class Migration(migrations.Migration):

    dependencies = [
        ('django_datawatch', '0004_resultassigneduser_resultassignedgroup_and_more'),
    ]

    operations = [
        migrations.RunPython(migrate_new_assigned_fields),
    ]



from unittest import mock

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.test.testcases import TestCase

from django_datawatch.base import BaseCheck
from django_datawatch.models import Result, ResultStatusHistory

User = get_user_model()

class BaseCheckTestCase(TestCase):
    def setUp(self) -> None:
        self.check = type('FakeCheck', (BaseCheck,), dict(slug='fake_check'))()
        self.fake_payload = type('FakePayload', (object,), dict(pk=1))()

    def test_save_tracks_status_change_on_initial_creation(self):
        result = self.check.save(self.fake_payload, Result.STATUS.ok)

        history_qs = ResultStatusHistory.objects.all()
        self.assertEqual(history_qs.count(), 1)

        history = history_qs.get()
        self.assertEqual(history.result, result)
        self.assertEqual(history.from_status, None)
        self.assertEqual(history.to_status, result.status)

    def test_save_does_not_track_same_status_change(self):
        result1 = self.check.save(self.fake_payload, Result.STATUS.ok)

        history_qs = ResultStatusHistory.objects.all()
        self.assertEqual(history_qs.count(), 1)

        history = history_qs.get()
        self.assertEqual(history.result, result1)
        self.assertEqual(history.to_status, result1.status)

        result2 = self.check.save(self.fake_payload, Result.STATUS.ok)
        self.assertEqual(result1, result2)
        self.assertEqual(history_qs.count(), 1)

        history = history_qs.get()
        self.assertEqual(history.result, result2)
        self.assertEqual(history.to_status, result2.status)

    def test_save_tracks_different_status_changes(self):
        result1 = self.check.save(self.fake_payload, Result.STATUS.ok)

        history_qs = ResultStatusHistory.objects.all()
        self.assertEqual(history_qs.count(), 1)

        history = history_qs.get()
        self.assertEqual(history.to_status, result1.status)
        self.assertEqual(history.to_status, result1.status)

        result2 = self.check.save(self.fake_payload, Result.STATUS.critical)
        self.assertEqual(history_qs.count(), 2)

        latest_history = history_qs.latest('created')
        self.assertEqual(latest_history.to_status, result2.status)
        self.assertEqual(latest_history.to_status, result2.status)

    def test_save_sets_groups_and_users(self):
        groups = [Group.objects.create(name=f'group_{i}') for i in range(3)]
        users = [User.objects.create_user(**{User.USERNAME_FIELD: f'user_{i}'}) for i in range(3)]

        self.check.get_assigned_groups = mock.Mock(return_value=groups)
        self.check.get_assigned_users = mock.Mock(return_value=users)

        result = self.check.save(self.fake_payload, Result.STATUS.ok)

        self.assertEqual(result.assigned_groups.count(), 3)
        for group in groups:
            self.assertTrue(group in result.assigned_groups.all())
        self.assertEqual(result.assigned_users.count(), 3)
        for user in users:
            self.assertTrue(user in result.assigned_users.all())

        # Update with no assigned users/groups
        self.check.get_assigned_groups = mock.Mock(return_value=None)
        self.check.get_assigned_users = mock.Mock(return_value=None)
        result = self.check.save(self.fake_payload, Result.STATUS.ok)

        self.assertEqual(result.assigned_groups.count(), 0)
        self.assertEqual(result.assigned_users.count(), 0)

    def test_unique_groups(self):
        group = Group.objects.create(name='group')
        self.check.get_assigned_groups = mock.Mock(return_value=[group, group])

        result = self.check.save(self.fake_payload, Result.STATUS.ok)

        # Result should be created with only one group
        self.assertEqual(Result.objects.count(), 1)
        self.assertEqual(result.assigned_groups.count(), 1)
        self.assertEqual(result.assigned_groups.first(), group)

    def test_unique_users(self):
        user = User.objects.create_user(**{User.USERNAME_FIELD: 'test_user'})
        self.check.get_assigned_users = mock.Mock(return_value=[user, user])

        result = self.check.save(self.fake_payload, Result.STATUS.ok)

        # Result should be created with only one user
        self.assertEqual(Result.objects.count(), 1)
        self.assertEqual(result.assigned_users.count(), 1)
        self.assertEqual(result.assigned_users.first(), user)



from six import iteritems

from django.db.models.base import Model
from django.views.generic.edit import FormMixin
from django.views.generic.list import ListView


class FilteredListView(FormMixin, ListView):
    SESSION_KEY = 'datawatch_search_term'

    def get_form_kwargs(self):
        form_data = self.request.session.get(self.SESSION_KEY, None)
        return {
          'initial': self.get_initial(),
          'prefix': self.get_prefix(),
          'data': self.request.GET or form_data
        }

    def get(self, request, *args, **kwargs):
        self.object_list = self.get_queryset()

        form = self.get_form(self.get_form_class())
        self.object_list = form.filter_queryset(request, self.object_list)
        if form.is_valid():
            form_data = dict()
            for k, v in iteritems(form.cleaned_data):
                if isinstance(v, Model):
                    v = v.pk
                form_data[k] = v
            self.request.session[self.SESSION_KEY] = dict(form_data)

        context = self.get_context_data(form=form, object_list=self.object_list)
        return self.render_to_response(context)



class BaseBackend(object):
    def enqueue(self, slug, run_async=True):
        raise NotImplementedError('enqueue not implemented')

    def refresh(self, slug, run_async=True):
        raise NotImplementedError('refresh not implemented')

    def run(self, slug, identifier, run_async=True, user_forced_refresh=False):
        raise NotImplementedError('run not implemented')



import logging

from django.core.exceptions import ObjectDoesNotExist

from django_datawatch.backends.base import BaseBackend
from django_datawatch.models import Result
from django_datawatch.datawatch import datawatch

logger = logging.getLogger(__name__)


class Backend(BaseBackend):
    def enqueue(self, slug, run_async=True):
        check = self._get_check_instance(slug)
        if not check:
            return

        try:
            for payload in check.generate():
                if payload is None:
                    continue
                datawatch.get_backend().run(check.slug, check.get_identifier(payload))
        except NotImplementedError as e:
            logger.error(e)

    def refresh(self, slug, run_async=True):
        for result in Result.objects.filter(slug=slug):
            datawatch.get_backend().run(result.slug, result.identifier)

    def run(self, slug, identifier, run_async=True, user_forced_refresh=False):
        check = self._get_check_instance(slug)
        if not check:
            return

        try:
            payload = check.get_payload(identifier)
        except NotImplementedError as e:
            logger.error(e)
            return
        except ObjectDoesNotExist:
            Result.objects.filter(slug=slug, identifier=identifier).delete()
            return

        # refresh has been forced by a user from the web view
        if user_forced_refresh:
            check.user_forced_refresh_hook(payload=payload)

        check.handle(payload)

    def _get_check_instance(self, slug):
        check_class = datawatch.get_check_class(slug)
        if not check_class:
            return None
        return check_class()



from django import forms
from django.contrib.auth import get_user_model
from django.core.validators import MaxValueValidator
from django.utils.translation import gettext_lazy as _
from model_utils.choices import Choices

from django_datawatch.models import Result
from django_datawatch.datawatch import datawatch


class ResultFilterForm(forms.Form):
    STATUS_CHOICES = Choices((0, 'all', _('All')), (1, 'failed', _('Failed')))
    CHECK_CHOICES = [('', _('All'))] + [(obj().slug, obj().get_title())
                                        for obj in datawatch.get_all_registered_checks()]

    user = forms.ModelChoiceField(queryset=get_user_model().objects.all().order_by('first_name', 'last_name'),
                                  label=_('User'), required=False)
    status = forms.TypedChoiceField(coerce=int, choices=STATUS_CHOICES, label=_('Status'),
                                    initial=STATUS_CHOICES.failed)
    check = forms.ChoiceField(choices=CHECK_CHOICES, label=_('Check'), required=False)

    def __init__(self, user, group_filter=None, **kwargs):
        super(ResultFilterForm, self).__init__(**kwargs)

        if not group_filter:
            group_filter = dict()

        self.fields['user'].initial = user
        self.fields['user'].queryset = self.fields['user'].queryset.filter(**group_filter)

    def filter_queryset(self, request, queryset):
        # default values if form has not been submitted
        if not self.is_bound:
            return queryset.failed().unacknowledged().for_user(request.user)
        # form has been submitted with invalid values
        if not self.is_valid():
            return queryset.none()

        if self.cleaned_data['user']:
            queryset = queryset.for_user(self.cleaned_data['user'])
        if self.cleaned_data['status']:
            if self.cleaned_data['status'] == self.STATUS_CHOICES.failed:
                queryset = queryset.failed().unacknowledged()
        if self.cleaned_data['check']:
            queryset = queryset.filter(slug=self.cleaned_data['check'])

        return queryset.distinct()


class AcknowledgeForm(forms.ModelForm):
    days = forms.IntegerField(min_value=1, max_value=365, label=_('Days to acknowledge'))

    class Meta:
        model = Result
        fields = ['days', 'acknowledged_reason']

    def __init__(self, user, **kwargs):
        self.user = user
        super(AcknowledgeForm, self).__init__(**kwargs)
        max_acknowledge = self.instance.get_check_instance().get_max_acknowledge()
        if max_acknowledge:
            self.fields['days'].validators.append(MaxValueValidator(max_acknowledge))

    def save(self, commit=True):
        self.instance.acknowledge(user=self.user, days=self.cleaned_data.get('days'),
                                  reason=self.cleaned_data['acknowledged_reason'], commit=commit)
        return self.instance



from unittest import mock

from django.db import transaction
from django.test.testcases import TestCase, override_settings

from django_datawatch.backends.base import BaseBackend
from django_datawatch.datawatch import datawatch, run_checks
from django_datawatch.base import BaseCheck
from django_datawatch.models import Result


@datawatch.register
class CheckTriggerUpdate(BaseCheck):
    model_class = Result
    trigger_update = dict(foobar=Result)

    def get_foobar_payload(self, instance):
        return instance

    def get_identifier(self, payload):
        return payload.pk

    def check(self, payload):
        return payload


@datawatch.register
class CheckTriggerUpdateList(BaseCheck):
    model_class = Result
    trigger_update = dict(foobar=Result)

    def get_foobar_payload(self, instance):
        return [instance, Result(pk=51945, slug="51945")]

    def get_identifier(self, payload):
        return payload.pk

    def check(self, payload):
        return payload


class TriggerUpdateTestCase(TestCase):
    @override_settings(DJANGO_DATAWATCH_RUN_SIGNALS=True)
    @mock.patch('django_datawatch.datawatch.DatawatchHandler.update_related')
    def test_setting_run_signals_true(self, mock_update):
        run_checks(sender='sender', instance='instance', created=None, raw=None,
                   using=None)
        mock_update.assert_called_once_with('sender', 'instance', None)

    @override_settings(DJANGO_DATAWATCH_RUN_SIGNALS=False)
    @mock.patch('django_datawatch.datawatch.DatawatchHandler.update_related')
    def test_setting_run_signals_false(self, mock_update):
        run_checks(sender=None, instance=None, created=None, raw=None,
                   using=None)
        mock_update.assert_not_called()

    @override_settings(DJANGO_DATAWATCH_RUN_SIGNALS=True)
    @mock.patch('django_datawatch.datawatch.DatawatchHandler.get_backend')
    def test_update_related_calls_backend(self, mock_get_backend):
        backend = mock.Mock(spec=BaseBackend)
        mock_get_backend.return_value = backend

        with self.captureOnCommitCallbacks() as callbacks:
            datawatch.update_related(sender=Result, instance=Result(pk=143243, slug="143243"))

        self.assertEquals(3, len(callbacks))

        parameters = [
            dict(slug='django_datawatch.tests.test_trigger_update.CheckTriggerUpdate', identifier=143243, run_async=True),
            dict(slug='django_datawatch.tests.test_trigger_update.CheckTriggerUpdateList', identifier=143243, run_async=True),
            dict(slug='django_datawatch.tests.test_trigger_update.CheckTriggerUpdateList', identifier=51945, run_async=True),
        ]

        for callback in callbacks:
            callback()
            parameter_list = parameters.pop(0)
            backend.run.assert_called_once_with(**parameter_list)
            backend.reset_mock()



import datetime
import pytz
from unittest import mock

from celery.schedules import crontab
from django.test.testcases import TestCase
from django.conf import settings
from time_machine import travel

from django_datawatch.base import BaseCheck
from django_datawatch.datawatch import datawatch, Scheduler


class CheckRunEvery(BaseCheck):
    run_every = crontab(hour=0, minute=0)


class CheckNoRunEvery(BaseCheck):
    run_every = None


class SchedulerTestCase(TestCase):
    @mock.patch('django_datawatch.tests.test_scheduler.CheckNoRunEvery.run')
    def test_skip_no_run_every(self, mock_run):
        datawatch.get_all_registered_checks = mock.MagicMock(return_value=[
            CheckNoRunEvery])
        scheduler = Scheduler()
        scheduler.get_last_executions = mock.MagicMock(return_value={})
        scheduler.run_checks()
        self.assertFalse(mock_run.called)

    @mock.patch('django_datawatch.tests.test_scheduler.CheckRunEvery.run')
    def test_no_previous_execution(self, mock_run):
        datawatch.get_all_registered_checks = mock.MagicMock(return_value=[
            CheckRunEvery])
        scheduler = Scheduler()
        scheduler.get_last_executions = mock.MagicMock(return_value={})
        scheduler.run_checks()
        self.assertTrue(mock_run.called)

    @travel(datetime.datetime(2016, 12, 1, 0, 0))
    @mock.patch('django_datawatch.tests.test_scheduler.CheckRunEvery.run')
    def test_execution_in_past(self, mock_run):
        datawatch.get_all_registered_checks = mock.MagicMock(return_value=[
            CheckRunEvery])
        scheduler = Scheduler()

        scheduler.get_last_executions = mock.MagicMock(return_value={
            'django_datawatch.tests.test_scheduler.CheckRunEvery':
                datetime.datetime(2016, 1, 1, 0, 0, 0, 0, pytz.timezone(
                    settings.TIME_ZONE)),
        })
        scheduler.run_checks()
        self.assertTrue(mock_run.called)

    @travel(datetime.datetime(2016, 1, 1, 0, 0))
    @mock.patch('django_datawatch.tests.test_scheduler.CheckRunEvery.run')
    def test_execution_in_future(self, mock_run):
        datawatch.get_all_registered_checks = mock.MagicMock(return_value=[
            CheckRunEvery])
        scheduler = Scheduler()

        scheduler.get_last_executions = mock.MagicMock(return_value={
            'django_datawatch.tests.test_scheduler.CheckRunEvery':
                datetime.datetime(2016, 12, 1, 0, 0, 0, 0, pytz.timezone(
                    settings.TIME_ZONE)),
        })
        scheduler.run_checks()
        self.assertFalse(mock_run.called)

    @travel(datetime.datetime(2016, 12, 1, 0, 0))
    @mock.patch('django_datawatch.tests.test_scheduler.CheckRunEvery.run')
    def test_execution_in_future_and_force(self, mock_run):
        datawatch.get_all_registered_checks = mock.MagicMock(return_value=[
            CheckRunEvery])
        scheduler = Scheduler()

        scheduler.get_last_executions = mock.MagicMock(return_value={
            'django_datawatch.tests.test_scheduler.CheckRunEvery':
                datetime.datetime(2016, 12, 1, 0, 0, 0, 0, pytz.timezone(
                    settings.TIME_ZONE)),
        })
        scheduler.run_checks(force=True)
        self.assertTrue(mock_run.called)



from django.test.testcases import TestCase

from django_datawatch.base import BaseCheck
from django_datawatch.datawatch import datawatch


class CheckImplementationTestCase(TestCase):
    pass


def test_generator(check_instance):
    def test(self):
        if not hasattr(check_instance, 'slug'):
            self.fail('{check} has no slug'.format(
                check=check_instance.__class__.__name__))
        self.assertIsInstance(
            check_instance, BaseCheck,
            '{slug} is not derived from BaseCheck'.format(
                slug=check_instance.slug))
        check_dict = check_instance.__class__.__dict__

        # check general implementation
        if 'check' not in check_dict:
            self.fail('{slug} must implement the check method'.format(
                slug=check_instance.slug))
        if 'get_identifier' not in check_dict:
            self.fail('{slug} must implement the get_identifier method'.format(
                slug=check_instance.slug))

        # generate must be implemented if task should be running periodically
        if check_dict.get('run_every', None) is not None:
            if 'generate' not in check_dict:
                self.fail('{slug} must implement the generate method'.format(
                    slug=check_instance.slug))

        # a resolver method must be implemented for every update trigger
        if check_dict.get('trigger_update', None) is not None:
            for key, value in check_dict['trigger_update'].items():
                method_name = 'get_%s_payload' % key
                if method_name not in check_dict:
                    self.fail(
                        '{slug} must implement a resolver method for every'
                        ' trigger_update, {method} is missing'.format(
                            slug=check_instance.slug, method=method_name))

    return test


datawatch.autodiscover_checks()
for check in datawatch.get_all_registered_checks():
    check_instance = check()
    test_name = 'test_{module}_{check}'.format(
        module=check_instance.__module__,
        check=check_instance.__class__.__name__)
    test = test_generator(check_instance)
    setattr(CheckImplementationTestCase, test_name, test)



# Generated by Django 4.2.9 on 2024-01-16 00:07

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_datawatch', '0005_migrate_new_assigned_fields'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='result',
            name='assigned_to_group',
        ),
        migrations.RemoveField(
            model_name='result',
            name='assigned_to_user',
        ),
    ]



from django_datawatch.models import Result, CheckExecution
from celery import shared_task
from celery.utils.log import get_task_logger

from django_datawatch.backends import synchronous
from django_datawatch.datawatch import Scheduler

logger = get_task_logger(__name__)


@shared_task
def django_datawatch_enqueue(slug, *args, **kwargs):
    logger.debug("enqueuing checks for %s", slug)
    synchronous.Backend().enqueue(slug=slug)


@shared_task
def django_datawatch_refresh(slug, *args, **kwargs):
    logger.debug("refreshing check results for %s", slug)
    synchronous.Backend().refresh(slug=slug)


@shared_task
def django_datawatch_run(slug, identifier, user_forced_refresh=False, *args, **kwargs):
    logger.debug(
        "running check %s for identifier %s (forced refresh %s)",
        slug,
        identifier,
        user_forced_refresh,
    )
    synchronous.Backend().run(
        slug=slug, identifier=identifier, user_forced_refresh=user_forced_refresh
    )


@shared_task
def django_datawatch_scheduler(*args, **kwargs):
    Scheduler().run_checks(force=False)


@shared_task
def datawatch_cleanup(*args, **kwargs):
    results = Result.objects.ghost_results()
    check_names = list(results.distinct().values_list("slug", flat=True))
    results.delete()
    check_names_str = "\n".join(check_names)
    logger.info(f"{len(check_names)} results have been deleted:\n{check_names_str}")

    check_executions = CheckExecution.objects.ghost_executions()
    check_names = list(check_executions.distinct().values_list("slug", flat=True))
    check_executions.delete()
    check_names_str = "\n".join(check_names)
    logger.info(f"{len(check_names)} check executions have been deleted:\n{check_names_str}")



import logging
from contextlib import contextmanager
from typing import Optional, List

from django import forms
from django.contrib.auth.models import AbstractUser, Group
from django.db import transaction
from django.utils import timezone

from django_datawatch.models import Result, CheckExecution, ResultStatusHistory
from django_datawatch.datawatch import datawatch, make_model_uid

logger = logging.getLogger(__name__)


@contextmanager
def track_status_history(slug, identifier, new_status):
    result = Result.objects.filter(slug=slug, identifier=identifier).only('id', 'status').first()
    current_status = getattr(result, 'status', None)
    yield
    if current_status != new_status:
        if result is None:
            result = Result.objects.only('id').get(slug=slug, identifier=identifier)
        ResultStatusHistory.objects.create(from_status=current_status, to_status=new_status, result_id=result.id)


class DatawatchCheckSkipException(Exception):
    pass

class BaseCheckForm(forms.Form):
    def save(self, instance):
        instance.config = self.cleaned_data
        instance.save(update_fields=['config'])
        return instance


class CheckResponse(object):
    def __init__(self):
        super(CheckResponse, self).__setattr__('_datastore', {})
        super(CheckResponse, self).__setattr__('_status', Result.STATUS.unknown)

    def __setattr__(self, name, value):
        self._datastore[name] = value

    def __getattr__(self, name):
        if name not in self._datastore:
            return None
        return self._datastore[name]

    def get_data(self):
        return self._datastore

    def set_status(self, status):
        super(CheckResponse, self).__setattr__('_status', status)

    def get_status(self):
        return self._status


class BaseCheck(object):
    """
    Any check should inherits from `BaseCheck` and should implements `.generate(self)`
    and `.check(self, payload)` methods.

    Optionally, you can implements `.get_assigned_users(self, payload)` (resp. `.get_assigned_groups(self, payload)`)
    to define to which user(s) (resp. group(s)) the system had to assign the check result.
    """

    config_form = None
    title = ''
    max_acknowledge = None
    run_every = None
    trigger_update = dict()
    model_class = None

    def __init__(self):
        self.slug = datawatch.get_slug(self.__module__, self.__class__.__name__)

    def user_forced_refresh_hook(self, payload):
        """
        gets only executed when the refresh has been forced by a user
        from the web view
        """
        pass

    def run(self):
        datawatch.get_backend().enqueue(slug=self.slug)
        CheckExecution.objects.update_or_create(slug=self.slug, defaults=dict(
            last_run=timezone.now()))

    def refresh(self):
        datawatch.get_backend().refresh(slug=self.slug)

    def handle(self, payload):
        # get old result
        old_status = None
        result = Result.objects.filter(slug=self.slug, identifier=self.get_identifier(payload)).first()
        if result:
            old_status = result.status

        # run check
        try:
            response = self.check(payload)
        except DatawatchCheckSkipException:
            if result:
                result.delete()
            return

        # save check result
        status = response.get_status()
        unacknowledge = old_status in [Result.STATUS.warning, Result.STATUS.critical] and status == Result.STATUS.ok
        self.save(payload=payload, status=status, data=response.get_data(), unacknowledge=unacknowledge)

    def get_config(self, payload):
        try:
            check_result = Result.objects.get(slug=self.slug, identifier=self.get_identifier(payload))

            # check has a configuration
            if check_result.config:
                return check_result.config
        except Result.DoesNotExist:
            pass

        # get default config from form initial values
        form = self.get_form_class()()
        return {name: field.initial for name, field in form.fields.items()}

    def get_form(self, payload):
        return self.get_form_class()(**self.get_config(payload))

    def get_form_class(self):
        return self.config_form

    def save(self, payload, status, data=None, unacknowledge=False):
        # build default data
        defaults = dict(status=status, data=data, payload_description=self.get_payload_description(payload))

        if unacknowledge:
            defaults.update(dict(acknowledged_by=None, acknowledged_at=None, acknowledged_until=None))

        with transaction.atomic():
            with track_status_history(self.slug, self.get_identifier(payload), status):
                # save the check
                dataset, created = Result.objects.update_or_create(
                    slug=self.slug, identifier=self.get_identifier(payload),
                    defaults=defaults)

                # set assigned users and groups
                if groups := self.get_assigned_groups(payload, status):
                    dataset.assigned_groups.set(groups)
                else:
                    dataset.assigned_groups.clear()

                if users := self.get_assigned_users(payload, status):
                    dataset.assigned_users.set(users)
                else:
                    dataset.assigned_users.clear()

        return dataset

    def get_trigger_update_uid_map(self):
        mapping = {}
        for method_name, model in self.trigger_update.items():
            mapping[make_model_uid(model)] = 'get_%s_payload' % method_name
        return mapping

    def generate(self):
        """
        yield items to run check for
        """
        raise NotImplementedError(".generate() must be overridden")

    def check(self, payload):
        """
        :param payload: the payload to run the check for
        :return:
        """
        raise NotImplementedError(".check() must be overridden")

    def get_identifier(self, payload):
        return payload.pk

    def get_payload(self, identifier):
        return self.model_class.objects.get(pk=identifier)

    def register(self, check_class):
        pass

    def get_payload_description(self, payload):
        return str(payload)

    def format_result_data(self, result):
        return ''

    def get_assigned_users(self, payload, result) -> Optional[List[AbstractUser]]:
        return None

    def get_assigned_groups(self, payload, result) -> Optional[List[Group]]:
        return None

    def get_context_data(self, result):
        return dict()

    def get_title(self):
        return self.title

    def get_template_name(self):
        if hasattr(self, 'template_name'):
            return self.template_name
        return None

    def get_max_acknowledge(self):
        return self.max_acknowledge



from django.db import models
from django.db.models.aggregates import Count
from django.db.models.expressions import Case, When, Value
from django.db.models.query_utils import Q
from django.utils import timezone

from django_datawatch.datawatch import datawatch


class ResultQuerySet(models.QuerySet):
    def for_user(self, user):
        # Should only check assigned groups if result has no assigned users
        return self.filter(
            Q(assigned_users=user)
            | Q(assigned_users__isnull=True, assigned_groups__in=user.groups.all())
            | Q(assigned_users__isnull=True, assigned_groups__isnull=True)
        ).distinct()

    def failed(self):
        return self.exclude(status__in=(self.model.STATUS.unknown, self.model.STATUS.ok))

    def ok(self):
        return self.filter(status=self.model.STATUS.ok)

    def unknown(self):
        return self.filter(status=self.model.STATUS.unknown)

    def unacknowledged(self):
        return self.exclude(acknowledged_until__gt=timezone.now())

    def with_status_name(self):
        case = Case(output_field=models.CharField())
        for status_value in self.model.STATUS._db_values:
            case.cases.append(
                When(status=status_value, then=Value(str(self.model.STATUS[status_value]))),
            )
        return self.annotate(status_name=case)

    def get_stats(self):
        return self.values('status').annotate(amount=Count('id')).with_status_name()

    def ghost_results(self):
        """
        :return: results that do not have checks anymore (check has been deleted)
        """
        return self.exclude(slug__in=datawatch.get_all_registered_check_slugs())


class CheckExecutionQuerySet(models.QuerySet):
    def ghost_executions(self):
        """
        :return: check executions that do not have checks anymore (check has been deleted)
        """
        return self.exclude(slug__in=datawatch.get_all_registered_check_slugs())



from django.core.management.base import BaseCommand

from django_datawatch.datawatch import Scheduler


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument(
            '--force',
            action='store_true',
            dest='force',
            default=False,
            help='Execute all checks.',
        )
        parser.add_argument(
            '--slug',
            dest='slug',
            default=None,
            help='Slug of check to refresh, all checks will be refreshed if slug is not provided',
        )

    def handle(self, force, slug, *args, **options):
        Scheduler().run_checks(force=force, slug=slug)



import os
import re
from os.path import join, dirname
from setuptools import setup, find_packages


def get_version(*file_paths):
    """Retrieves the version from the given path"""
    filename = os.path.join(os.path.dirname(__file__), *file_paths)
    version_file = open(filename).read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


def long_description():
    """Return long description from README.rst if it's present
    because it doesn't get installed."""
    try:
        return open(join(dirname(__file__), 'README.md')).read()
    except IOError:
        return ''


setup(
    name='django-datawatch',
    packages=find_packages(exclude=['example*']),
    version=get_version("django_datawatch", "__init__.py"),
    description='Django Datawatch runs automated data checks in your Django installation',
    author='Jens Nistler <opensource@jensnistler.de>, Bogdan Radko <bogdan.radko@regiohelden.de>',
    author_email='opensource@regiohelden.de',
    long_description=long_description(),
    long_description_content_type='text/markdown',
    install_requires=[
        'celery>=5.0.0',
        'Django>=3.2,<5.0',
        'django-extensions>=3.0.0',
        'django-model-utils>=4.2.0',
        'python-dateutil>=2.8.0',
    ],
    license='MIT',
    url='https://github.com/RegioHelden/django-datawatch',
    download_url='',
    keywords=['django', 'monitoring', 'datawatch', 'check', 'checks'],
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: System :: Monitoring",
    ]
)



from typing import Optional
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.test.testcases import TestCase

from django_datawatch.models import Result

User = get_user_model()


class ResultQuerySetTestCase(TestCase):
    def _make_users(self, usernames: list):
        users = []
        for username in usernames:
            users.append(User(**{User.USERNAME_FIELD: username}))
        return User.objects.bulk_create(users)

    def _make_groups(self, names: list):
        groups = []
        for name in names:
            groups.append(Group(name=name))
        return Group.objects.bulk_create(groups)

    def _make_result(
        self, slug, users: Optional[list] = None, groups: Optional[list] = None
    ):
        result = Result.objects.create(slug=slug, identifier="test_identifier")
        if users:
            result.assigned_users.set(users)
        if groups:
            result.assigned_groups.set(groups)
        return result

    def _assert_for_user_results(self, user, expected_results):
        queryset = Result.objects.for_user(user).order_by("pk")
        self.assertEqual(queryset.count(), len(expected_results))
        self.assertEqual(list(queryset), sorted(expected_results, key=lambda x: x.pk))

    def test_for_user(self):
        groups = self._make_groups(["g1", "g2", "g3"])
        users = self._make_users(["u1", "u2", "u3"])

        group_with_user = groups[1]
        group_without_users = groups[2]
        group_with_users = groups[0]

        user_with_group = users[0]
        user_without_group = users[1]
        user_with_groups = users[2]

        user_with_group.groups.set([group_with_users])
        user_with_groups.groups.set([group_with_users, group_with_user])

        # No group and no user (u1, u2, u3)
        res_no_user_no_group = self._make_result("test1")
        # One group but no user (u3)
        res_with_group_no_user = self._make_result("test2", groups=[group_with_user])
        # One user but no group (u1)
        res_with_user_no_group = self._make_result("test3", users=[user_with_group])
        # Multiple groups but no user (u3)
        res_with_groups_no_user = self._make_result(
            "test4", groups=[group_with_user, group_without_users]
        )
        # Multiple users but no groups (u1, u3)
        res_with_users_no_group = self._make_result(
            "test5", users=[user_with_group, user_with_groups]
        )
        # One group and one user (u2)
        res_with_group_and_user = self._make_result(
            "test6", users=[user_without_group], groups=[group_with_user]
        )
        # Multiple groups and one user (u2)
        res_with_groups_and_user = self._make_result(
            "test7",
            users=[user_without_group],
            groups=[group_with_user, group_without_users],
        )
        # One group and multiple users (u1, u3)
        res_with_group_and_users = self._make_result(
            "test8", users=[user_with_group, user_with_groups], groups=[group_with_user]
        )
        self._assert_for_user_results(
            user_with_group,
            [
                res_no_user_no_group,
                res_with_user_no_group,
                res_with_users_no_group,
                res_with_group_and_users,
            ],
        )
        self._assert_for_user_results(
            user_without_group,
            [res_no_user_no_group, res_with_group_and_user, res_with_groups_and_user],
        )
        self._assert_for_user_results(
            user_with_groups,
            [
                res_no_user_no_group,
                res_with_group_no_user,
                res_with_groups_no_user,
                res_with_users_no_group,
                res_with_group_and_users,
            ],
        )



# Generated by Django 1.10.2 on 2018-08-07 15:08
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_datawatch', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='result',
            name='assigned_to_group',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='auth.Group'),
        ),
        migrations.AlterField(
            model_name='result',
            name='assigned_to_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='assigned_to_user', to=settings.AUTH_USER_MODEL),
        ),
    ]



# Generated by Django 3.2.7 on 2021-09-14 10:04

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    dependencies = [
        ('django_datawatch', '0002_auto_20180807_1508'),
    ]

    operations = [
        migrations.CreateModel(
            name='ResultStatusHistory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('from_status', models.IntegerField(choices=[(0, 'Unknown'), (1, 'OK'), (2, 'Warning'), (3, 'Critical')], null=True, verbose_name='From status')),
                ('to_status', models.IntegerField(choices=[(0, 'Unknown'), (1, 'OK'), (2, 'Warning'), (3, 'Critical')], verbose_name='To status')),
                ('result', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='status_history', related_query_name='status_history', to='django_datawatch.result', verbose_name='Result')),
            ],
            options={
                'verbose_name': 'Result status history',
                'verbose_name_plural': 'Result status history',
            },
        ),
    ]



from django.core.management.base import BaseCommand

from django_datawatch.datawatch import datawatch


class Command(BaseCommand):
    def handle(self, *args, **options):
        for slug in datawatch.get_all_registered_check_slugs():
            print(slug)



from django.apps import AppConfig
from django.contrib import admin
from django_datawatch.datawatch import datawatch


class DjangoDatawatchConfig(AppConfig):
    name = 'django_datawatch'
    verbose_name = "Django datawatch"

    def ready(self):
        super(DjangoDatawatchConfig, self).ready()

        datawatch.autodiscover_checks()
        admin.autodiscover()
